/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_MPI_PARALLEL_EFFICIENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_MPI_PARALLEL_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridMPICommunicationEfficiencyTest.h"
#include "BSPOPHybridMPILoadBalanceTest.h"

namespace bscanalysis
{
class BSPOPHybridOMPParallelEfficiencyTest;
class BSPOPHybridMPIParallelEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class BSPOPHybridOMPParallelEfficiencyTest;
private:

    cube::Metric* non_mpi_time;
    cube::Metric* max_runtime;

    cube::list_of_metrics max_runtime_metrics;

    void
    calculate( const cube::list_of_cnodes& );

protected:

    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "MPI Parallel Efficiency (MPE) reveals the inefficiency in MPI processes. \n"
            "MPE can be computed directly or as a product of MPI Load Balance Efficiency and MPI Communication Efficiency :\n"
            "\tMPE = avg(time outside of MPI) / max(runtime) \n"
            "\t    = MPI Load Balance Efficiency x MPI Communication Efficiency";
        std::string not_active_text =
            "MPI Parallel Efficiency (MPE) metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridMPIParallelEfficiencyTest(   cube::CubeProxy*  );

    virtual
    ~BSPOPHybridMPIParallelEfficiencyTest()
    {
    };
// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
